/*----------------------------------------------------------------------- 
 * 
 * File Name: VectorSequenceFactories.c
 * 
 * Author: Finn, L. S.
 * 
 * Revision: $Id: VectorSequenceFactories.m4,v 1.1 2000/04/17 23:22:48 jolien Exp $
 * 
 *----------------------------------------------------------------------- 
 * 
 * SYNOPSIS 
 * void TYPECODECreateVectorSequence(
 *     Status                  *status,
 *     TYPEVectorSequence     **vseq,
 *     CreateVectorSequenceIn  *in
 *     );
 * void TYPECODEDestroyVectorSequence(
 *     Status              *status,
 *     TYPEVectorSequence **vector
 *     );
 * 
 * DESCRIPTION 
 * Create/destroy a TYPEVectorSequence object. 
 * 
 * DIAGNOSTICS 
 * Illegal sequence length, illegal vectorLength, vseq == NULL, 
 * *vseq != NULL, malloc failure 
 *
 * CALLS
 * LALMalloc
 * LALFree
 * 
 *-----------------------------------------------------------------------
 */

#include "LALStdlib.h"
#include "SeqFactories.h"

NRCSID( VECTORSEQUENCEFACTORIESC, "$Id: VectorSequenceFactories.m4,v 1.1 2000/04/17 23:22:48 jolien Exp $" );

















void ZCreateVectorSequence ( Status *status, COMPLEX16VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{
  /* 
   * Initialize status
   */
  INITSTATUS( status, "ZCreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, CREATEVECSEQ_EINPTR, CREATEVECSEQ_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          CREATEVECSEQ_ESLENGTH, CREATEVECSEQ_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          CREATEVECSEQ_EVLENGTH, CREATEVECSEQ_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, CREATEVECSEQ_EVPTR, CREATEVECSEQ_MSGEVPTR);
  ASSERT (*vseq == NULL, status, CREATEVECSEQ_EUPTR, CREATEVECSEQ_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( COMPLEX16VectorSequence * ) LALMalloc( sizeof( COMPLEX16VectorSequence ) );
  ASSERT (*vseq != NULL, status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( COMPLEX16 );
    (*vseq)->data = ( COMPLEX16 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















void ZDestroyVectorSequence ( Status *status, COMPLEX16VectorSequence **vseq )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "ZDestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, DESTROYVECSEQ_EVPTR, DESTROYVECSEQ_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, DESTROYVECSEQ_EUPTR, DESTROYVECSEQ_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          DESTROYVECSEQ_EDPTR, DESTROYVECSEQ_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















void CCreateVectorSequence ( Status *status, COMPLEX8VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{
  /* 
   * Initialize status
   */
  INITSTATUS( status, "CCreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, CREATEVECSEQ_EINPTR, CREATEVECSEQ_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          CREATEVECSEQ_ESLENGTH, CREATEVECSEQ_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          CREATEVECSEQ_EVLENGTH, CREATEVECSEQ_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, CREATEVECSEQ_EVPTR, CREATEVECSEQ_MSGEVPTR);
  ASSERT (*vseq == NULL, status, CREATEVECSEQ_EUPTR, CREATEVECSEQ_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( COMPLEX8VectorSequence * ) LALMalloc( sizeof( COMPLEX8VectorSequence ) );
  ASSERT (*vseq != NULL, status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( COMPLEX8 );
    (*vseq)->data = ( COMPLEX8 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















void CDestroyVectorSequence ( Status *status, COMPLEX8VectorSequence **vseq )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "CDestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, DESTROYVECSEQ_EVPTR, DESTROYVECSEQ_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, DESTROYVECSEQ_EUPTR, DESTROYVECSEQ_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          DESTROYVECSEQ_EDPTR, DESTROYVECSEQ_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















void DCreateVectorSequence ( Status *status, REAL8VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{
  /* 
   * Initialize status
   */
  INITSTATUS( status, "DCreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, CREATEVECSEQ_EINPTR, CREATEVECSEQ_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          CREATEVECSEQ_ESLENGTH, CREATEVECSEQ_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          CREATEVECSEQ_EVLENGTH, CREATEVECSEQ_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, CREATEVECSEQ_EVPTR, CREATEVECSEQ_MSGEVPTR);
  ASSERT (*vseq == NULL, status, CREATEVECSEQ_EUPTR, CREATEVECSEQ_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( REAL8VectorSequence * ) LALMalloc( sizeof( REAL8VectorSequence ) );
  ASSERT (*vseq != NULL, status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( REAL8 );
    (*vseq)->data = ( REAL8 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















void DDestroyVectorSequence ( Status *status, REAL8VectorSequence **vseq )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "DDestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, DESTROYVECSEQ_EVPTR, DESTROYVECSEQ_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, DESTROYVECSEQ_EUPTR, DESTROYVECSEQ_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          DESTROYVECSEQ_EDPTR, DESTROYVECSEQ_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















void SCreateVectorSequence ( Status *status, REAL4VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{
  /* 
   * Initialize status
   */
  INITSTATUS( status, "SCreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, CREATEVECSEQ_EINPTR, CREATEVECSEQ_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          CREATEVECSEQ_ESLENGTH, CREATEVECSEQ_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          CREATEVECSEQ_EVLENGTH, CREATEVECSEQ_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, CREATEVECSEQ_EVPTR, CREATEVECSEQ_MSGEVPTR);
  ASSERT (*vseq == NULL, status, CREATEVECSEQ_EUPTR, CREATEVECSEQ_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( REAL4VectorSequence * ) LALMalloc( sizeof( REAL4VectorSequence ) );
  ASSERT (*vseq != NULL, status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( REAL4 );
    (*vseq)->data = ( REAL4 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















void SDestroyVectorSequence ( Status *status, REAL4VectorSequence **vseq )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "SDestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, DESTROYVECSEQ_EVPTR, DESTROYVECSEQ_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, DESTROYVECSEQ_EUPTR, DESTROYVECSEQ_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          DESTROYVECSEQ_EDPTR, DESTROYVECSEQ_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















void I2CreateVectorSequence ( Status *status, INT2VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{
  /* 
   * Initialize status
   */
  INITSTATUS( status, "I2CreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, CREATEVECSEQ_EINPTR, CREATEVECSEQ_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          CREATEVECSEQ_ESLENGTH, CREATEVECSEQ_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          CREATEVECSEQ_EVLENGTH, CREATEVECSEQ_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, CREATEVECSEQ_EVPTR, CREATEVECSEQ_MSGEVPTR);
  ASSERT (*vseq == NULL, status, CREATEVECSEQ_EUPTR, CREATEVECSEQ_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( INT2VectorSequence * ) LALMalloc( sizeof( INT2VectorSequence ) );
  ASSERT (*vseq != NULL, status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( INT2 );
    (*vseq)->data = ( INT2 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















void I2DestroyVectorSequence ( Status *status, INT2VectorSequence **vseq )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "I2DestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, DESTROYVECSEQ_EVPTR, DESTROYVECSEQ_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, DESTROYVECSEQ_EUPTR, DESTROYVECSEQ_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          DESTROYVECSEQ_EDPTR, DESTROYVECSEQ_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















void I4CreateVectorSequence ( Status *status, INT4VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{
  /* 
   * Initialize status
   */
  INITSTATUS( status, "I4CreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, CREATEVECSEQ_EINPTR, CREATEVECSEQ_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          CREATEVECSEQ_ESLENGTH, CREATEVECSEQ_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          CREATEVECSEQ_EVLENGTH, CREATEVECSEQ_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, CREATEVECSEQ_EVPTR, CREATEVECSEQ_MSGEVPTR);
  ASSERT (*vseq == NULL, status, CREATEVECSEQ_EUPTR, CREATEVECSEQ_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( INT4VectorSequence * ) LALMalloc( sizeof( INT4VectorSequence ) );
  ASSERT (*vseq != NULL, status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( INT4 );
    (*vseq)->data = ( INT4 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















void I4DestroyVectorSequence ( Status *status, INT4VectorSequence **vseq )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "I4DestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, DESTROYVECSEQ_EVPTR, DESTROYVECSEQ_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, DESTROYVECSEQ_EUPTR, DESTROYVECSEQ_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          DESTROYVECSEQ_EDPTR, DESTROYVECSEQ_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















void I8CreateVectorSequence ( Status *status, INT8VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{
  /* 
   * Initialize status
   */
  INITSTATUS( status, "I8CreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, CREATEVECSEQ_EINPTR, CREATEVECSEQ_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          CREATEVECSEQ_ESLENGTH, CREATEVECSEQ_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          CREATEVECSEQ_EVLENGTH, CREATEVECSEQ_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, CREATEVECSEQ_EVPTR, CREATEVECSEQ_MSGEVPTR);
  ASSERT (*vseq == NULL, status, CREATEVECSEQ_EUPTR, CREATEVECSEQ_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( INT8VectorSequence * ) LALMalloc( sizeof( INT8VectorSequence ) );
  ASSERT (*vseq != NULL, status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( INT8 );
    (*vseq)->data = ( INT8 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















void I8DestroyVectorSequence ( Status *status, INT8VectorSequence **vseq )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "I8DestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, DESTROYVECSEQ_EVPTR, DESTROYVECSEQ_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, DESTROYVECSEQ_EUPTR, DESTROYVECSEQ_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          DESTROYVECSEQ_EDPTR, DESTROYVECSEQ_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















void U2CreateVectorSequence ( Status *status, UINT2VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{
  /* 
   * Initialize status
   */
  INITSTATUS( status, "U2CreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, CREATEVECSEQ_EINPTR, CREATEVECSEQ_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          CREATEVECSEQ_ESLENGTH, CREATEVECSEQ_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          CREATEVECSEQ_EVLENGTH, CREATEVECSEQ_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, CREATEVECSEQ_EVPTR, CREATEVECSEQ_MSGEVPTR);
  ASSERT (*vseq == NULL, status, CREATEVECSEQ_EUPTR, CREATEVECSEQ_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( UINT2VectorSequence * ) LALMalloc( sizeof( UINT2VectorSequence ) );
  ASSERT (*vseq != NULL, status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( UINT2 );
    (*vseq)->data = ( UINT2 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















void U2DestroyVectorSequence ( Status *status, UINT2VectorSequence **vseq )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "U2DestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, DESTROYVECSEQ_EVPTR, DESTROYVECSEQ_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, DESTROYVECSEQ_EUPTR, DESTROYVECSEQ_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          DESTROYVECSEQ_EDPTR, DESTROYVECSEQ_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















void U4CreateVectorSequence ( Status *status, UINT4VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{
  /* 
   * Initialize status
   */
  INITSTATUS( status, "U4CreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, CREATEVECSEQ_EINPTR, CREATEVECSEQ_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          CREATEVECSEQ_ESLENGTH, CREATEVECSEQ_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          CREATEVECSEQ_EVLENGTH, CREATEVECSEQ_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, CREATEVECSEQ_EVPTR, CREATEVECSEQ_MSGEVPTR);
  ASSERT (*vseq == NULL, status, CREATEVECSEQ_EUPTR, CREATEVECSEQ_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( UINT4VectorSequence * ) LALMalloc( sizeof( UINT4VectorSequence ) );
  ASSERT (*vseq != NULL, status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( UINT4 );
    (*vseq)->data = ( UINT4 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















void U4DestroyVectorSequence ( Status *status, UINT4VectorSequence **vseq )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "U4DestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, DESTROYVECSEQ_EVPTR, DESTROYVECSEQ_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, DESTROYVECSEQ_EUPTR, DESTROYVECSEQ_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          DESTROYVECSEQ_EDPTR, DESTROYVECSEQ_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















void U8CreateVectorSequence ( Status *status, UINT8VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{
  /* 
   * Initialize status
   */
  INITSTATUS( status, "U8CreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, CREATEVECSEQ_EINPTR, CREATEVECSEQ_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          CREATEVECSEQ_ESLENGTH, CREATEVECSEQ_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          CREATEVECSEQ_EVLENGTH, CREATEVECSEQ_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, CREATEVECSEQ_EVPTR, CREATEVECSEQ_MSGEVPTR);
  ASSERT (*vseq == NULL, status, CREATEVECSEQ_EUPTR, CREATEVECSEQ_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( UINT8VectorSequence * ) LALMalloc( sizeof( UINT8VectorSequence ) );
  ASSERT (*vseq != NULL, status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( UINT8 );
    (*vseq)->data = ( UINT8 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















void U8DestroyVectorSequence ( Status *status, UINT8VectorSequence **vseq )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "U8DestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, DESTROYVECSEQ_EVPTR, DESTROYVECSEQ_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, DESTROYVECSEQ_EUPTR, DESTROYVECSEQ_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          DESTROYVECSEQ_EDPTR, DESTROYVECSEQ_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















void CHARCreateVectorSequence ( Status *status, CHARVectorSequence **vseq, CreateVectorSequenceIn *in ) 
{
  /* 
   * Initialize status
   */
  INITSTATUS( status, "CHARCreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, CREATEVECSEQ_EINPTR, CREATEVECSEQ_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          CREATEVECSEQ_ESLENGTH, CREATEVECSEQ_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          CREATEVECSEQ_EVLENGTH, CREATEVECSEQ_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, CREATEVECSEQ_EVPTR, CREATEVECSEQ_MSGEVPTR);
  ASSERT (*vseq == NULL, status, CREATEVECSEQ_EUPTR, CREATEVECSEQ_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( CHARVectorSequence * ) LALMalloc( sizeof( CHARVectorSequence ) );
  ASSERT (*vseq != NULL, status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( CHAR );
    (*vseq)->data = ( CHAR * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















void CHARDestroyVectorSequence ( Status *status, CHARVectorSequence **vseq )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "CHARDestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, DESTROYVECSEQ_EVPTR, DESTROYVECSEQ_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, DESTROYVECSEQ_EUPTR, DESTROYVECSEQ_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          DESTROYVECSEQ_EDPTR, DESTROYVECSEQ_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}


















void CreateVectorSequence ( Status *status, REAL4VectorSequence **vseq, CreateVectorSequenceIn *in ) 
{
  /* 
   * Initialize status
   */
  INITSTATUS( status, "CreateVectorSequence", VECTORSEQUENCEFACTORIESC );	

  /* Check input structure: report if NULL */

  ASSERT (in != NULL, status, CREATEVECSEQ_EINPTR, CREATEVECSEQ_MSGEINPTR);
      
  /* Check sequence length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->length > 0, status,
          CREATEVECSEQ_ESLENGTH, CREATEVECSEQ_MSGESLENGTH);

  /* Check vector length: report error if 0 
   * Use of unsigned for length means we can't check if negative
   * length was passed
   */

  ASSERT (in->vectorLength > 0, status,
          CREATEVECSEQ_EVLENGTH, CREATEVECSEQ_MSGEVLENGTH); 

  /* 
   * Check return structure: If return pointer does not point to a
   *    valid pointer then report an error 
   */

  ASSERT (vseq != NULL, status, CREATEVECSEQ_EVPTR, CREATEVECSEQ_MSGEVPTR);
  ASSERT (*vseq == NULL, status, CREATEVECSEQ_EUPTR, CREATEVECSEQ_MSGEUPTR);

  /*
   * Allocate pointer
   */

  *vseq = ( REAL4VectorSequence * ) LALMalloc( sizeof( REAL4VectorSequence ) );
  ASSERT (*vseq != NULL, status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);

  (*vseq)->length = 0;	/* length 0 until storage allocated */
  (*vseq)->vectorLength = 0; /* vector length 0 until storage allocated */
  (*vseq)->data   = NULL;	/* NULL data until allocated */

  /* 
   * Allocate storage 
   */

  {
    size_t tlength;
    tlength = in->vectorLength * in->length * sizeof( REAL4 );
    (*vseq)->data = ( REAL4 * ) LALMalloc (tlength);
  }

  if (NULL == (*vseq)->data)
  {
    /* Must free storage pointed to by *vseq */
    LALFree ((void *) *vseq);
    ABORT (status, CREATEVECSEQ_EMALLOC, CREATEVECSEQ_MSGEMALLOC);
  }
 
  /* Set length, vectorLength if storage allocated */

  (*vseq)->length = in->length;	
  (*vseq)->vectorLength = in->vectorLength;

  /* We be done: Normal exit */

  RETURN (status);
}
















void DestroyVectorSequence ( Status *status, REAL4VectorSequence **vseq )
{
  /* 
   * Initialize status
   */

  INITSTATUS( status, "DestroyVectorSequence", VECTORSEQUENCEFACTORIESC );
      
  /* 
   * Check vseq: is it non-NULL?
   */

  ASSERT (vseq != NULL, status, DESTROYVECSEQ_EVPTR, DESTROYVECSEQ_MSGEVPTR); 

  /* 
   * Check vseq: does it point to non-NULL?
   */

  ASSERT (*vseq != NULL,status, DESTROYVECSEQ_EUPTR, DESTROYVECSEQ_MSGEUPTR);

  /*
   * Check data in vseq: does it point to non-NULL?
   */

  ASSERT ((*vseq)->data != NULL, status,
          DESTROYVECSEQ_EDPTR, DESTROYVECSEQ_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vseq)->data ); /* free allocated data */
  LALFree ( *vseq );	      /* free vseq struct itself */

  *vseq = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}

